module Qsimpx
  ! simple Q transform

  use adj_mod

  implicit none
  integer, private :: nf, nx, nq, iif, iz, ix, iq 
  real   , private :: f0, df, x0, dx, q0, dq, x, qc, qx, qz, f, z, wt
  logical, private :: lint, image
  integer, private :: nz
  real   , private :: dz, z0 , sx, const

  contains

  subroutine Qsimpx_init ( f0_in,df_in,x0_in,dx_in,q0_in,dq_in,z0_in, dz_in,nf_in, &
    nx_in,nq_in, nz_in,  sx_in, const_in,image_in, lint_in )
    real    :: f0_in, df_in, x0_in, dx_in, q0_in, dq_in
    integer :: nx_in, nq_in, nf_in
    logical :: lint_in, image_in
    integer :: nz_in
    real    :: dz_in, z0_in, sx_in, const_in

    f0 = f0_in 
    df = df_in 
    x0 = x0_in 
    dx = dx_in 
    q0 = q0_in 
    dq = dq_in 
    nf = nf_in 
    nx = nx_in 
    nq = nq_in 
    lint = lint_in
    nz = nz_in
    z0 = z0_in
    dz = dz_in
    sx = sx_in
    const=const_in
    image=image_in
  end subroutine

  function Qsimpx_lop ( adj, add, modl, data) result(stat)
    integer            :: stat 
    logical,intent(in) :: adj,add 
    real,dimension(:)  :: modl,data 
    call adjnull      (adj,add,modl,data )
    call Qsimpx_lop2(adj,add,modl,data )
    stat=0
  end function 

  subroutine Qsimpx_lop2(adj,add,modl,data)
    logical,intent(in) :: adj,add 
    real, dimension (nz,nq)  :: modl 
    real, dimension (nf,nx)  :: data 
    real    :: ff
    integer :: iff

    do iq = 1,nq  
     ! q loop
      qc = (q0 + (iq-1) * dq)/const  
      do ix = 1,nx 
        ! offset loop
	if (image) then
      		  x = (x0 + (ix-1) * dx -sx)
	else
		  x = (x0 + (ix-1) * dx -sx)/2.
	end if

        do iz = 1,nz  
          ! zero-offset traveltime loop
          z = z0 + (iz-1) * dz 
          qx = abs( qc * x)                     ! normalized offset
	  qz = abs( qc * z)
          f  = 2.*sqrt( qz * qz + qx * qx)        ! NMO



          if ( lint) then
            ! Do linear interpolation...
           ff= (f-f0)/df
           iff=ff
           iif=1+iff  
           wt = ff-iff             

            if ( iif > nf-1 .OR. iif<1) then     
              exit                  ! out of the range
            end if
          else
            ! ...or nearest-neighbor.
            iif = 1.5 + (f-f0)/df               
            wt = 1.0
            if ( iif > nf) then
              exit                  ! out of the range
            end if
          end if
                           
          ! (t,x)-dependent weighting
          if ( adj) then
            if ( lint) then
	      modl(iz,iq) = modl(iz,iq) + data(iif,ix)*(1.-wt) + data(iif+1,ix)*wt
            else
              modl(iz,iq) = modl(iz,iq) + data(iif,ix)*wt
            end if
          else
            if ( lint) then
              data(iif  ,ix) = data(iif  ,ix) + modl(iz,iq)*(1.-wt)
              data(iif+1,ix) = data(iif+1,ix) + modl(iz,iq)*    wt
            else
              data(iif,ix) = data(iif,ix) + modl(iz,iq)*wt
            end if
          end if
        end do 
       ! End time loop
      end do 
    ! End offset loop
    end do 
   ! End zero-offset traveltime loop

  end subroutine
 
  subroutine Qsimpx_close()
  end subroutine

end module
! end module
