!!! Q transform in data domain--Yi Shen Feb 21, 2012
!!!	param:
!!!		var=[500.]   !!! Variance of the spectra (sigma^2); result will be sensitive to variance
!!!		vel=[2560.]  !!! Velocity above the reflector; Velocity maybe changed through dispersion
!!!		sx=[5000]    !!! Source location
!!!	adj:
!!!		nz=[100] z0=[0.] dz=[1.]  !!! depth
!!!		nq=[] q0=[] dq=[]  !!! Inverse of Q
!!! 	fwd:
!!!		nx=[nz] x0=[0.] dx=[1.]  !!! receiver location
!!!		nf=[] f0=[0.] df=[0.5]   !!! central frequency shift (fs-fr)



program Qtran                       ! Q transform
  use sep
  use Qsimpx
  use smallsolver
  use cgstep_mod

  implicit none

  integer                          ::  iadj, iinv, ilint, niter, stat
  real                             :: o1,d1, o2,d2, x0,dx, q0,dq, dz, z0, f0, df
  real, dimension(:), allocatable  :: data, model, error, mydata
  logical                          :: adj, inv, lint, image
  real                             :: temp2(5)
  real 				   :: fs, var, vel, const, pi, sx
  integer			   :: nz, n1, n2, nf, nq, nx

  call sep_init()

  call from_param( "adj",  iadj, 1);   adj  = (iadj==1)
  call from_param( "inv",  iinv, 0);   inv  = (iinv==1)
  call from_param( "lint", ilint, 0);  lint = (ilint==1)
  call from_param("var",var,500.)
  call from_param("vel",vel,2500.)
  call from_param("sx",sx, 0.) !!! source location
  call from_param('image',image,.false.)

  if (adj) then 

        call from_param("nz",nz,100)
        call from_param("dz",dz,1.)
        call from_param("z0",z0,0.)

        call from_param ( "nq", nq       )
        call from_param(  "q0", q0       ) 
        call from_param(  "dq", dq       )  

  	call from_history(  "n2", nx )
  	call from_history(  "o2", x0 ) 
  	call from_history(  "d2", dx )  

	call from_history(  "n1", nf  )
  	call from_history(  "o1", f0  )
  	call from_history(  "d1", df  ) 

        call to_history ("label2","Inverse of Q")
        n2 = nq; o2 = q0; d2 = dq
        call to_history ("label1", "depth(m)")
        n1 = nz; o1 = z0; d1 = dz
 

  else
	call from_param(  "nx", nx, nz)
  	call from_param(  "x0", x0, 0.)
  	call from_param(  "dx", dx, 1.) 

	call from_param(  "nf", nf       )
  	call from_param(  "f0", f0, 0.   )
  	call from_param(  "df", df, 0.5  ) 

  	call from_history(  "n2", nq )
  	call from_history(  "o2", q0 ) 
  	call from_history(  "d2", dq )  

  	call from_history("n1",nz)
  	call from_history("o1",z0)
  	call from_history("d1",dz)

        call to_history ("label2","distance(m)")
        n2 = nx; o2 = x0; d2 = dx
        call to_history ("label1", "central frequency shift")
        n1 = nf; o1 = f0; d1 = df


  end if

  call to_history ("o1", o1)
  call to_history ("d1", d1)
  call to_history ("n1", n1)

  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)
  
  call from_param( "niter", niter, 1)
  if (exist_file ("error")) call to_history ("n1",niter,"error")
  call sep_close()
  
  allocate( data(nf*nx), model(nz*nq), mydata(nf*nx) )
  pi=3.14159265
  if (image) then
	 const=vel*vel/2./pi/var
  else
	 const=vel/pi/var
  end if
  
  call Qsimpx_init( f0, df, x0, dx, q0, dq, z0, dz, nf, nx, nq, nz,sx,const,image,lint)

  if( adj) then  
     allocate (error (niter))
     call sep_read (data)
     if( inv) then		              ! Do LS inverse...	
        stat = Qsimpx_lop( adj, .false., model, data)
	call solver(oper=Qsimpx_lop, solv=cgstep, x=model,dat=data,niter=niter)
     else                                     ! Or simple adjoint.
        stat = Qsimpx_lop( adj, .false., model, data)
     end if
     call sep_write( model)
     if (exist_file ("error")) call sep_write (error/dot_product (data,data), "error")
     deallocate (error)
  else
     call sep_read (model)
     stat = Qsimpx_lop (adj, .false., model, data)
     call sep_write (data)
  end if

  deallocate( model, data, mydata)

  call exit (0)
end program Qtran
