!!! Compute central frequency -- Yi Shen, Feb 21, 2012
!!! fpeak.x < input.H fpeak_loc=fpeak_loc.H dfsfr=dfsfr.H fs=50. fac=1 norm=0 > output.H
!!! 	output.H:	2d fr
!!!	fpeak_loc.H:	1d fr
!!!	dfsfr.H		2d fs-fr
!!!	fs:		[50] central frequency of the source; used in 2d fs-fr; 
!!!	fac:		[1] scale factor; If (fs-fr) is too small, enlarge it by fac factor; used in 2d fs-fr
!!!	norm		[0] whether to normalized the spectral; normalize=1 to keep the amplitude of the central frequency the same at different offset; used in 2d fr and fs-fr

program fpeak
  use sep
  use util_axisIO

  implicit none

 
  integer 			:: n2,i2,i1,n1,n3
  real,  allocatable 		:: spectra(:,:),fpeak2(:,:),fpeak1(:), deltf(:,:), mean(:)
  real				:: d1,d2,d3,o2, o1, oo1
  real				:: top,bot,loc_real
  real				:: loc
  real    			:: ll1, wt1
  integer 			:: ill1,il1
  real    			:: ll2, wt2
  integer 			:: ill2,il2
  type(myaxis)  		:: axis_dummy,axis_n2,axis_nn1,axis_nn2
  real 				:: fs, totmean
  integer			:: fac
  integer			:: norm


  call sep_init()

  call from_history ('n1',n1)
  call from_history ("d1",d1)
  call from_history ("o1",o1)
  call from_history ('n2', n2,1)
  call from_history ('d2',d2)
  call from_history ('o2',o2)
  call from_history ('n3', n3,1)
  call from_history ('d3',d3,1.0)
 
  call from_param('fs', fs, 50.)
  call from_param('fac', fac, 1)
  call from_param('norm',norm,0 )

  allocate(spectra(n1,n2),fpeak2(n1,n2),fpeak1(n2),deltf(fac*n1,n2),mean(n2))
  call sep_read(spectra)
  
  fpeak1=0; deltf=0
  oo1=(fs-((n1-1)*d1+o1))	!! the origin of fs-fr
  totmean=sum(spectra)/n1/n2


  do i2=1,n2
	top=0
	bot=0
	do i1=1,n1
		top=top+(o1+(i1-1)*d1)*spectra(i1,i2)
		bot=bot+spectra(i1,i2)

	end do
	mean(i2) = sum(spectra(:,i2))/n1
	loc=top/bot
	loc_real=top/bot
	ll1 = (loc-o1)/d1
        ill1=ll1
        il1=1+ill1  
        wt1 = ll1-ill1	!!!linear intepolation	


!!! 2d fr
	if (norm==.false.) then
 		fpeak2(il1,i2)=mean(i2)*(1.-wt1)
 		fpeak2(il1+1,i2)=mean(i2)*wt1
	else
 		fpeak2(il1,i2)= totmean*(1.-wt1)
 		fpeak2(il1+1,i2)= totmean*wt1
	end if

!!! 1d fr
	fpeak1(i2)=loc_real


!!! 2d fs-fr
	ll2 = fac*(fs-loc-oo1)/d1
        ill2=ll2
        il2=1+ill2  
        wt2 = ll2-ill2	!!!linear intepolation	
	if (norm==.false.) then
	  	deltf(il2,i2)=mean(i2)*(1.-wt2)
  		deltf(il2+1,i2)=mean(i2)*wt2
	else
 		deltf(il2,i2)=totmean*(1.-wt2)
 		deltf(il2+1,i2)=totmean*wt2
	end if
  end do
  
  call to_history('n1',n1)
  call to_history('n2',n2)

  call to_history('d1',d1)
  call to_history('d2',d2)
  
  call to_history('o1',o1)
  call to_history('o2',o2)

  call set_axis(axis_dummy,1,1.0,1.0,"")
  call houtaxis(axis_dummy,2,'fpeak_loc')
  call set_axis(axis_n2,n2,d2,o2,"")
  call houtaxis(axis_n2,1,'fpeak_loc')



  call set_axis(axis_nn1,fac*n1,d1,oo1*fac,"")
  call houtaxis(axis_nn1,1,'dfsfr')
  call set_axis(axis_nn2,n2,d2,o2,"")
  call houtaxis(axis_nn2,2,'dfsfr')

  call sep_write(fpeak2)
  call sep_write(fpeak1,'fpeak_loc')
  call sep_write(deltf,'dfsfr')

  deallocate(fpeak2,spectra,fpeak1,deltf,mean)

  call sep_close()
end program

